@@ -5,10 +5,9 @@
 
     The PHP domain. Based off of the rubydomain by SHIBUKAWA Yoshiki
 
-    :copyright: Copyright 2011 by Mark Story
+    :copyright: Copyright 2017 by Mark Story
     :license: BSD, see LICENSE for details.
 """
-
 import re
 
 from docutils import nodes
@@ -24,7 +23,6 @@ from sphinx.util.compat import Directive
 from sphinx.util.docfields import Field, GroupedField, TypedField
 
 
-
 php_sig_re = re.compile(
     r'''^ ([\w.]*\:\:)?          # class name(s)
           (\$?\w+)  \s*          # thing name
@@ -34,8 +32,6 @@ php_sig_re = re.compile(
           ''', re.VERBOSE)
 
 
-php_paramlist_re = re.compile(r'([\[\],])')  # split at '[', ']' and ','
-
 NS = '\\'
 
 separators = {
@@ -55,6 +51,55 @@ separators = {
 php_separator = re.compile(r"(\w+)?(?:[:]{2})?")
 
 
+def _pseudo_parse_arglist(signode, arglist):
+    # type: (addnodes.desc_signature, unicode) -> None
+    """"Parse" a list of arguments separated by commas.
+    Arguments can have "optional" annotations given by enclosing them in
+    brackets.  Currently, this will split at any comma, even if it's inside a
+    string literal (e.g. default argument value).
+
+    This function comes from sphinx.domains.python.
+    """
+    paramlist = addnodes.desc_parameterlist()
+    stack = [paramlist]
+    try:
+        for argument in arglist.split(','):
+            argument = argument.strip()
+            ends_open = ends_close = 0
+            while argument.startswith('['):
+                stack.append(addnodes.desc_optional())
+                stack[-2] += stack[-1]
+                argument = argument[1:].strip()
+            while argument.startswith(']'):
+                stack.pop()
+                argument = argument[1:].strip()
+            while argument.endswith(']') and not argument.endswith('[]'):
+                ends_close += 1
+                argument = argument[:-1].strip()
+            while argument.endswith('['):
+                ends_open += 1
+                argument = argument[:-1].strip()
+            if argument:
+                stack[-1] += addnodes.desc_parameter(argument, argument)
+            while ends_open:
+                stack.append(addnodes.desc_optional())
+                stack[-2] += stack[-1]
+                ends_open -= 1
+            while ends_close:
+                stack.pop()
+                ends_close -= 1
+        if len(stack) != 1:
+            raise IndexError
+    except IndexError:
+        # if there are too few or too many elements on the stack, just give up
+        # and treat the whole argument list as one argument, discarding the
+        # already partially populated paramlist node
+        signode += addnodes.desc_parameterlist()
+        signode[-1] += addnodes.desc_parameter(arglist, arglist)
+    else:
+        signode += paramlist
+
+
 def php_rsplit(fullname):
     items = [item for item in php_separator.findall(fullname)]
     return ''.join(items[:-2]), ''.join(items[1:-1])
@@ -123,6 +168,9 @@ class PhpObject(ObjectDescription):
         classname = self.env.temp_data.get('php:class')
         separator = separators[self.objtype]
 
+        # Method declared as Class::methodName
+        if not classname and '::' in name_prefix:
+            classname = name_prefix.rstrip('::')
         if self.objtype == 'global' or self.objtype == 'function':
             add_module = False
             modname = None
@@ -130,7 +178,7 @@ class PhpObject(ObjectDescription):
             fullname = name
         else:
             add_module = True
-            # name_prefix and a non-static method, means the classname was 
+            # name_prefix and a non-static method, means the classname was
             # repeated. Trim off the <class>::
             if name_prefix and self.objtype != 'staticmethod':
                 if name_prefix.startswith(classname):
@@ -186,26 +234,8 @@ class PhpObject(ObjectDescription):
                 signode += addnodes.desc_returns(retann, retann)
             return fullname, name_prefix
 
-        signode += addnodes.desc_parameterlist()
+        _pseudo_parse_arglist(signode, arglist)
 
-        stack = [signode[-1]]
-        for token in php_paramlist_re.split(arglist):
-            if token == '[':
-                opt = addnodes.desc_optional()
-                stack[-1] += opt
-                stack.append(opt)
-            elif token == ']':
-                try:
-                    stack.pop()
-                except IndexError:
-                    raise ValueError
-            elif not token or token == ',' or token.isspace():
-                pass
-            else:
-                token = token.strip()
-                stack[-1] += addnodes.desc_parameter(token, token)
-        if len(stack) != 1:
-            raise ValueError
         if retann:
             signode += addnodes.desc_returns(retann, retann)
         return fullname, name_prefix
@@ -217,7 +247,8 @@ class PhpObject(ObjectDescription):
         raise NotImplementedError('must be implemented in subclasses')
 
     def _is_class_member(self):
-        return self.objtype.startswith('method') or self.objtype.startswith('attr')
+        return (self.objtype.startswith('method') or
+                self.objtype.startswith('attr'))
 
     def add_target_and_index(self, name_cls, sig, signode):
         if self.objtype == 'global':
@@ -254,8 +285,7 @@ class PhpObject(ObjectDescription):
         indextext = self.get_index_text(modname, name_cls)
         if indextext:
             self.indexnode['entries'].append(('single', indextext,
-                                              fullname, fullname))
-
+                                              fullname, fullname, None))
 
 
 class PhpGloballevel(PhpObject):
@@ -269,6 +299,7 @@ class PhpGloballevel(PhpObject):
         else:
             return ''
 
+
 class PhpNamespacelevel(PhpObject):
     """
     Description of an object on namespace level (functions, constants).
@@ -336,6 +367,7 @@ class PhpClasslike(PhpObject):
         if self.names:
             self.env.temp_data['php:class'] = self.names[0][0]
 
+
 class PhpClassmember(PhpObject):
     """
     Description of a class member (methods, attributes).
@@ -379,6 +411,7 @@ class PhpClassmember(PhpObject):
         else:
             return ''
 
+
 class PhpNamespace(Directive):
     """
     Directive to start a new PHP namespace, which are similar to modules.
@@ -403,7 +436,8 @@ class PhpNamespace(Directive):
             env.docname, self.options.get('synopsis', ''),
             'deprecated' in self.options)
 
-        targetnode = nodes.target('', '', ids=['namespace-' + modname], ismod=True)
+        targetnode = nodes.target('', '', ids=['namespace-' + modname],
+                                  ismod=True)
         self.state.document.note_explicit_target(targetnode)
         ret = [targetnode]
 
@@ -412,7 +446,7 @@ class PhpNamespace(Directive):
         if not noindex:
             indextext = _('%s (namespace)') % modname
             inode = addnodes.index(entries=[('single', indextext,
-                                             'namespace-' + modname, modname)])
+                                             'namespace-' + modname, modname, None)])
             ret.append(inode)
         return ret
 
@@ -425,7 +459,7 @@ class PhpXRefRole(XRefRole):
         if not has_explicit_title:
             if title.startswith("::"):
                 title = title[2:]
-            target = target.lstrip('~') # only has a meaning for the title
+            target = target.lstrip('~')  # only has a meaning for the title
             # if the first character is a tilde, don't display the module/class
             # parts of the contents
             if title[0:1] == '~':
@@ -439,6 +473,7 @@ class PhpXRefRole(XRefRole):
 
         return title, target
 
+
 class PhpNamespaceIndex(Index):
     """
     Index subclass to provide the Php module index.
@@ -454,7 +489,7 @@ class PhpNamespaceIndex(Index):
         ignores = self.domain.env.config['modindex_common_prefix']
         ignores = sorted(ignores, key=len, reverse=True)
         # list of all modules, sorted by module name
-        modules = sorted(self.domain.data['namespaces'].iteritems(),
+        modules = sorted(self.domain.data['namespaces'].items(),
                          key=lambda x: x[0].lower())
         # sort out collapsable modules
         prev_modname = ''
@@ -503,10 +538,11 @@ class PhpNamespaceIndex(Index):
         collapse = len(modules) - num_toplevels < num_toplevels
 
         # sort by first letter
-        content = sorted(content.iteritems())
+        content = sorted(content.items())
 
         return content, collapse
 
+
 class PhpDomain(Domain):
     """PHP language domain."""
     name = 'php'
@@ -571,16 +607,23 @@ class PhpDomain(Domain):
     def resolve_xref(self, env, fromdocname, builder,
                      typ, target, node, contnode):
         if (typ == 'ns' or
-            typ == 'obj' and target in self.data['namespaces']):
-            docname, synopsis, deprecated = \
-                self.data['namespaces'].get(target, ('','','', ''))
+                typ == 'obj' and target in self.data['namespaces']):
+            docname, synopsis, deprecated = self.data['namespaces'].get(
+                target,
+                ('', '', '', '')
+            )
             if not docname:
                 return None
             else:
                 title = '%s%s' % (synopsis,
-                                    (deprecated and ' (deprecated)' or ''))
-                return make_refnode(builder, fromdocname, docname,
-                                    'namespace-' + target, contnode, title)
+                                  (deprecated and ' (deprecated)' or ''))
+                return make_refnode(
+                    builder,
+                    fromdocname,
+                    docname,
+                    'namespace-' + target,
+                    contnode,
+                    title)
         else:
             modname = node.get('php:namespace')
             clsname = node.get('php:class')
@@ -632,23 +675,23 @@ class PhpDomain(Domain):
             elif modname and modname + NS + name in objects:
                 newname = modname + NS + name
             elif modname and classname and \
-                     modname + NS + classname + '::' + name in objects:
+                    modname + NS + classname + '::' + name in objects:
                 newname = modname + NS + classname + '::' + name
             elif modname and classname and \
-                     modname + NS + classname + '::$' + name in objects:
+                    modname + NS + classname + '::$' + name in objects:
                 newname = modname + NS + classname + '::$' + name
             # special case: object methods
             elif type in ('func', 'meth') and '::' not in name and \
-                 'object::' + name in objects:
+                    'object::' + name in objects:
                 newname = 'object::' + name
         if newname is None:
             return None, None
         return newname, objects[newname]
 
     def get_objects(self):
-        for ns, info in self.data['namespaces'].iteritems():
+        for ns, info in self.data['namespaces'].items():
             yield (ns, ns, 'namespace', info[0], 'namespace-' + ns, 0)
-        for refname, (docname, type) in self.data['objects'].iteritems():
+        for refname, (docname, type) in self.data['objects'].items():
             yield (refname, refname, type, docname, refname, 1)
 
 
